﻿namespace Accountantix.Mobile.Core.Module.Services.General
{
    using System;
    using Accountantix.Mobile.Core.Module.Constants;
    using Accountantix.Mobile.Core.Module.Contracts.Services.General;
    using Accountantix.Mobile.Core.Module.ViewModels;
    using Accountantix.Mobile.Core.Module.Views.Dialogs;
    using Syncfusion.XForms.PopupLayout;
    using Xamarin.Forms;

    public class DialogService : IDialogService
    {
        #region Fields

        private readonly ITranslationService _translationService;

        private SfPopupLayout _popupDialog;

        #endregion

        #region Contructors

        public DialogService(ITranslationService translationService)
        {
            _translationService = translationService ?? throw new ArgumentNullException(nameof(translationService));

        }

        #endregion

        #region Interfaces Implementations

        public void Init()
        {
            _popupDialog = new SfPopupLayout
            {
                PopupView =
                {
                    ShowHeader = true,
                    ShowFooter = true,
                    ShowCloseButton = false,
                    AppearanceMode = AppearanceMode.TwoButton
                }
            };
        }

        #region Show

        public void Show(string title, string message, Action okCallback)
        {
            try
            {
                var vm = new DialogViewModel(okCallback) {Message = message};

                _popupDialog.PopupView.ShowHeader = true;
                _popupDialog.PopupView.ShowCloseButton = true;
                _popupDialog.PopupView.ShowFooter = true;
                _popupDialog.PopupView.HeaderTitle = title;
                _popupDialog.PopupView.AppearanceMode = AppearanceMode.OneButton;
                _popupDialog.PopupView.PopupStyle.BorderColor = Color.White;
                _popupDialog.PopupView.AcceptButtonText = _translationService.Get(Translations.DialogButtonOk);    
                _popupDialog.PopupView.AcceptCommand = vm.OkCommand;

                _popupDialog.PopupView.Opacity = 1;
                _popupDialog.Opacity = 1;
                _popupDialog.VerticalOptions = LayoutOptions.Center;
                _popupDialog.HorizontalOptions = LayoutOptions.Center;

                var templateView = new DataTemplate(() => new DialogView
                {
                    BindingContext = vm
                });

                _popupDialog.PopupView.ContentTemplate = templateView;

                _popupDialog.Show();
            }
            catch (Exception exception)
            {
                throw;
            }
        }

        #endregion

        #region ShowInfo

        public void ShowInfo(string message, Action okCallback)
        {
            Show(_translationService.Get(Translations.DialogTitleInfo), message, okCallback);
        }

        #endregion

        #region ShowWarning

        public void ShowWarning(string message, Action okCallback)
        {
            Show(_translationService.Get(Translations.DialogTitleWarning), message, okCallback);
        }

        #endregion

        #region ShowError

        public void ShowError(string message, Action okCallback)
        {
            Show(_translationService.Get(Translations.DialogTitleError), message, okCallback);
        }

        #endregion

        #region ShowConfirm

        public void ShowConfirm(string title, string question, Action okCallback, Action noCallback)
        {
            try
            {
                var vm = new ConfirmDialogViewModel(okCallback, noCallback) {Question = question};

                _popupDialog.PopupView.ShowHeader = true;
                _popupDialog.PopupView.ShowCloseButton = true;
                _popupDialog.PopupView.ShowFooter = true;
                _popupDialog.PopupView.HeaderTitle = title;
                _popupDialog.PopupView.AppearanceMode = AppearanceMode.OneButton;
                _popupDialog.PopupView.PopupStyle.BorderColor = Color.White;
                _popupDialog.PopupView.AcceptButtonText = _translationService.Get(Translations.DialogButtonOk);    
                _popupDialog.PopupView.AcceptCommand = vm.OkCommand;


                _popupDialog.PopupView.Opacity = 1;
                _popupDialog.Opacity = 1;
                _popupDialog.VerticalOptions = LayoutOptions.Center;
                _popupDialog.HorizontalOptions = LayoutOptions.Center;

                _popupDialog.PopupView.AcceptButtonText = _translationService.Get(Translations.DialogButtonOk);
                _popupDialog.PopupView.DeclineButtonText = _translationService.Get(Translations.DialogButtonCancel);
                _popupDialog.PopupView.AcceptCommand = vm.OkCommand;
                _popupDialog.PopupView.DeclineCommand = vm.CancelCommand;

                var templateView = new DataTemplate(() => new ConfirmDialogView
                {
                    BindingContext = vm
                });

                _popupDialog.PopupView.ContentTemplate = templateView;

                _popupDialog.Show();
            }
            catch (Exception exception)
            {
                throw;
            }
        }

        #endregion

        #region ShowBusyDialog

        public void ShowBusyDialog(string messages)
        {
            try
            {
                _popupDialog.PopupView.ShowHeader = false;
                _popupDialog.PopupView.ShowCloseButton = false;
                _popupDialog.PopupView.ShowFooter = false;

                _popupDialog.PopupView.Opacity = 0;
                _popupDialog.Opacity = 0;
                _popupDialog.VerticalOptions = LayoutOptions.FillAndExpand;
                _popupDialog.HorizontalOptions = LayoutOptions.FillAndExpand;

                _popupDialog.PopupView.PopupStyle.BorderColor = Color.Transparent;

                var templateView = new DataTemplate(() => new BusyDialogView
                {
                    BindingContext = new BusyDialogViewModel {IsBusy = true, Message = messages}
                });

                _popupDialog.PopupView.ContentTemplate = templateView;

                _popupDialog.Show();
            }
            catch (Exception exception)
            {
                throw;
            }
        }

        #endregion

        #region HideBusyDialog

        public void HideBusyDialog()
        {
            try
            {
                if (_popupDialog != null)
                {
                    _popupDialog.IsOpen = false;
                }
            }
            catch (Exception exception)
            {
                throw;
            }
        }

        #endregion

        #endregion
    }
}